%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%    Copyright (C) 2004 Adam Spiro and Aharon Bar-Hillel
%    An implementation of a spike sorting algorithm according to the paper:
%    "Spike Sorting: Bayesian Clustering of Non-Stationary Data" 
%    by Aharon Bar-Hillel, Adam Spiro and Eran Stark 
%    (http://www.cs.huji.ac.il/~aharonbh/papers/SpikeSort_NIPS04.ps)
%
%    This program is free software; you can redistribute it and/or modify
%    it under the terms of the GNU General Public License as published by
%    the Free Software Foundation; either version 2 of the License, or
%    (at your option) any later version.
%
%    This program is distributed in the hope that it will be useful,
%    but WITHOUT ANY WARRANTY; without even the implied warranty of
%    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
%    GNU General Public License for more details.
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

% points - the data points of one cluster from a solutions
% We calcluate how similar is the cluster to a gaussian. 
function [c,b] = GaussianSimilarity(points)

% actually, should take the 1st PCA of the points
for i=10:-1:1
	[a bx]=uniqFreq(round(points(:,1)/(10^i))');
	if (size(bx,2)>7)
        break;
	end
end

for i=10:-1:1
    [a2 by]=uniqFreq(round(points(:,2)/(10^i))');
	if (size(by,2)>7)
        break;
	end
end

cx=getScore(bx);
cy=getScore(by);

c = max([cx cy]);
if (c==cx)
    b=bx;
else
    b=by;
end

% We calculate the score by comparing the cluster to a gaussian (which can
% we afinitly moved (left or right).
function c = getScore(b);
s=b/max(b)*100;

%transform the points to all gaussian like possibilities
len=size(s,2);
comp(1,:)=sort(s);

for i=2:len
    comp(i,:)=comp(i-1,[2:len-i+2 1 len-i+3:len]);
end
comp=[comp' comp(:,end:-1:1)']';

% Calculate the scores
for i=1:size(comp,1)
	res(i)=0;
    sg=comp(i,:);
	for j=1:len
        res(i)=res(i)+(sg(j)-s(j))^2;
	end
	res(i)=res(i)/len;

end
c=min(res);

