%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%    Copyright (C) 2004 Adam Spiro and Aharon Bar-Hillel
%    An implementation of a spike sorting algorithm according to the paper:
%    "Spike Sorting: Bayesian Clustering of Non-Stationary Data" 
%    by Aharon Bar-Hillel, Adam Spiro and Eran Stark 
%    (http://www.cs.huji.ac.il/~aharonbh/papers/SpikeSort_NIPS04.ps)
%
%    This program is free software; you can redistribute it and/or modify
%    it under the terms of the GNU General Public License as published by
%    the Free Software Foundation; either version 2 of the License, or
%    (at your option) any later version.
%
%    This program is distributed in the hope that it will be useful,
%    but WITHOUT ANY WARRANTY; without even the implied warranty of
%    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
%    GNU General Public License for more details.
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%


% Get rid of small clusters that have a big covariance
% We Search for the cluster with the maximal covariance (except the
% backgroud) and transform it to the background in case it is a very small
% cluster.
[a b]=uniqFreq(NewLabels);
b1=zeros(1,max(a));
b1(a)=b;
b1(1)=0;
b1=b1/sum(b1);
% The threshold for the size of the small cluster
ind_small=find(b1<GParams.SizeOfSmallClusterWithLargeCov & b1>0);
for c=a
    DetCov(c)=det(cov(data(find(NewLabels==c),:)));
end
DetCov(1)=0; % not the background
[m1 m2]=max(DetCov); %find the maximal covariance

% If the cluster is very small and occurs for < 3 frames, remove it too
for ind=ind_small 
    max1=max(find(NewLabels==ind));
    min1=min(find(NewLabels==ind));
    
    if (max1-min1 < GParams.RareClusterRemoval)
        m2=[m2 ind];
    end
end

rem=intersect(m2,ind_small); % those who are both small and have large covariance

if ~isempty(rem)
    rem_ind=find(ismember(NewLabels,rem)==1);
    NewLabels(rem_ind)=1; % turn them into background
end
%%% end of get rid of small clusters
