%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%    Copyright (C) 2004 Adam Spiro and Aharon Bar-Hillel
%    An implementation of a spike sorting algorithm according to the paper:
%    "Spike Sorting: Bayesian Clustering of Non-Stationary Data" 
%    by Aharon Bar-Hillel, Adam Spiro and Eran Stark 
%    (http://www.cs.huji.ac.il/~aharonbh/papers/SpikeSort_NIPS04.ps)
%
%    This program is free software; you can redistribute it and/or modify
%    it under the terms of the GNU General Public License as published by
%    the Free Software Foundation; either version 2 of the License, or
%    (at your option) any later version.
%
%    This program is distributed in the hope that it will be useful,
%    but WITHOUT ANY WARRANTY; without even the implied warranty of
%    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
%    GNU General Public License for more details.
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

% Here we search for Small and Isolated clusters (S&I).
% For each one we found, we figure out whether it is a mirror or a
% largeScale clsuter.
% max_score - the score of the most seperable cluster
% max_labels - the labels of the data for the max_score
% Sol - the relevant solution
% Clust - the list of S&I clusters
% id - a list of the diagnosed clusters (see IsMirror for the details)

function [max_score, max_labels,Sol, Clust, id]= findMirrorLargeScale(data,Labels,GParams)

rand('state',1);
randn('state',1);

% Search for S&I
[max_score_x, max_labels_x, Sol_x, c_x]= seperateClust(data, Labels, GParams,50,1);
[max_score_y, max_labels_y, Sol_y, c_y]= seperateClust(data, Labels, GParams,50,2);
% if both have passed the threshold, take the one which covers more spikes
% that were originally labeled as background.
if (max_score_x>3.1 & max_score_y>3.1)
    if (c_x>2*c_y)
        max_score_y=0;
    else
        max_score_x=0;
    end
end
if (max_score_x>max_score_y)
    max_score=max_score_x;
    max_labels=max_labels_x;
    Sol=Sol_x;
else
    max_score=max_score_y;
    max_labels=max_labels_y;
    Sol=Sol_y;    
end

Ind=find(Labels==1);
% visual_k_means2(data(Ind,:),max_labels(Ind));
% visual_k_means2(data,max_labels_x);
% max_score_x
% visual_k_means2(data,max_labels_y);
% max_score_y
% visual_k_means2(data,max_labels);

[a1,b1]=uniqFreq(max_labels);
[a2,b2]=uniqFreq(max_labels(Ind));
b11=zeros(1,size(b1,2));
b11(a1)=b1; % to deal when not all labels occur in b2
b22=zeros(1,size(b1,2));
b22(a2)=b2; % to deal when not all labels occur in b2

c=find(b22./b11>0.25); % if the relation is small, it means that the spikes are already labeled in the original solution
d=find(b22>100);  % ignore very small clusters
Clust=intersect(c,d);
Clust=setdiff(Clust,[1]); % not the background

% Remove clusters that have a very big variance
a=fmap('det',[Sol(:,2)]);
a{1}=0; % ignore the background
[c d]=max(cell2mat(a));
Clust=setdiff(Clust,d);

if isempty(Clust)
    disp('Niether mirror nor large scale clusters were detected');
    id=[];
    return;
else
    for C=Clust
        disp(sprintf('Checking if cluster is mirror or large scale %d',C));
        id(C)=isMirror(data,Labels,max_labels,C);
    end
end

