%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%    Copyright (C) 2004 Adam Spiro and Aharon Bar-Hillel
%    An implementation of a spike sorting algorithm according to the paper:
%    "Spike Sorting: Bayesian Clustering of Non-Stationary Data" 
%    by Aharon Bar-Hillel, Adam Spiro and Eran Stark 
%    (http://www.cs.huji.ac.il/~aharonbh/papers/SpikeSort_NIPS04.ps)
%
%    This program is free software; you can redistribute it and/or modify
%    it under the terms of the GNU General Public License as published by
%    the Free Software Foundation; either version 2 of the License, or
%    (at your option) any later version.
%
%    This program is distributed in the hope that it will be useful,
%    but WITHOUT ANY WARRANTY; without even the implied warranty of
%    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
%    GNU General Public License for more details.
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

% Here we search for mirror clusters and large scale clusters.
% Mirror clusters are clusters which are mostly positive, and most of its
% points have a mirror points (relatively to the y axis).
% Large Scale clusters are ones which are identifiable only when viewing
% the data on a big windows, in our case, we take all the data and search
% for small seperable clusters.

function NewLabels = prepareLabelsWithMirrorLargeScale(data,GParams,pLabels)

NewLabels=pLabels;
pos=find(data(:,1)>0);

% Deal with mirrors of the original labelings, i.e, take the Labels we got
% so far and search for mirror clusters.
disp(sprintf('Removing mirrors from the original labeling'));
for C=uniqFreq(pLabels)
    orig_id=isMirror(data,pLabels,NewLabels,C);
    if(orig_id)==-1 % should be backgournd
        NewLabels(intersect(pos,find(NewLabels==C)))=1;
    end
    if (orig_id>0) % it has a twin mirror.
        NewLabels(intersect(pos,find(NewLabels==C)))=1;%max(NewLabels)+1;
    end
end
 
% Find the Clusters
[max_score, max_labels,Sol, Clust, id]=findMirrorLargeScale(data,NewLabels,GParams);

% Mirror without a single twin
Mind=find(id==-1); % The mirror clusters
Rind=find(ismember(max_labels,Mind)); % The indices of the mirror clusters
NewLabels(intersect(Rind,pos))=1;

% Mirror with a single twin
Mind=find(id>0);
Rind=find(ismember(max_labels,Mind)); % The indices of the mirror clusters
maxL=max(NewLabels);
NewLabels(intersect(pos,Rind))=1;%maxL+1;


% Large Scale
Mind=find(id==-2); % The mirror clusters
Rind=find(ismember(max_labels,Mind)); % The indices of the mirror clusters
maxL=max(NewLabels);
NewLabels(Rind)=maxL+1;

% Delete the clusters that reduced dramitically after the new labelings
% (this is done for cases such as having a mirror cluster which also has
% some points in the negative section. In this case we want to put them in
% the background).

[a1 b1]=uniqFreq(pLabels);
[a2 b2]=uniqFreq(NewLabels);
m=max([max(a1) max(a2)]);
b11=zeros(1,m);
b22=zeros(1,m);
b11(a1)=b1;
b22(a2)=b2;
inds=find(b22./b11<0.5); % if they reduced by more than this value, delete them.
inds2=find(ismember(NewLabels,inds));
NewLabels(inds2)=1;