
% k                    : the number of componenets.
% data                 : the data - every row is a data point.

% probabilities        : matrix of k*length(data) of the probabilities of observables.
%                        column i contain pdf of the hidden of observable i.

function [newParams]= maximize2(data,k,param,Probabilities,oth,EMParams)

global mm_flag;
global late_oracle;
global exit_flags;
global frozen_models;
global TOTAL_COV; %FOR ANDRE SPEECH STUFF

w=EMParams.w;   % weight of type 1

% d is the dimension of each data point, n is the number of data points.
[n,d]=size(data);

numberOfObservables= n; %number of observable data points
%in the data set.

obsProbabilities=Probabilities(oth,:);	

%modify to handle weights: W is row vector
if(~isempty(w))
    normalizer= w*obsProbabilities;
else
    normalizer= sum(obsProbabilities);
end

newParams{1,2}= zeros(d,d);%init just in case single cov mat flag.

% approximate the new weights ( initial condition for a gradint ascent )
if late_oracle
    naiveWeights=normalizer;		% weights without considering partition function
else
    naiveWeights= sum(Probabilities);	% Noam's old model : these are hidden weights, not observable weights 
    newWeights=naiveWeights/sum(naiveWeights);
end

verySmall= [];

%when diag cov mat flag is on we need only 2 points for estimation
%(not a good one!)
if(EMParams.diagonal_covmat_flag == 0)
    % check for model degradation
    if ~EMParams.single_cov_mat_flag
        if (mm_flag) 
            verySmall=find(normalizer<d+1);	
            % this is the condition for the cov_mat of a class to be in full rank
        else
            verySmall=find(normalizer<d+1);	% stam , no thought here.
        end
    else
        if (mm_flag)
            verySmall=find(normalizer==0);	% stam , no thought here.
        else
            verySmall=find(normalizer<0.5);	% stam , no thought here.
        end
    end
end

% don't delete the background model if there is one.
if EMParams.BgrModel 
    verySmall=setdiff(verySmall,k);
    if naiveWeights(end)<(1e-6)*sum(naiveWeights)         % don't let the background model drop beyond this size.
        naiveWeights(end)=(1e-6)*sum(naiveWeights);         
    end
end

if any(verySmall)    %a small models was found
    if EMParams.remove_models_flag		% remove the small models : this will
        % probably cause an increase in the ll next round and hence a stop.
        rem=setdiff(1:k,verySmall); 
        k=k-length(verySmall);
        if k>0
            obsProbabilities=obsProbabilities(:,rem);
            Probabilities=Probabilities(:,rem);
            naiveWeights=naiveWeights(rem);
            normalizer=normalizer(rem);
            disp(sprintf('Model %d has been removed\n',verySmall));
            exit_flags=bitor(exit_flags,4);
        else
            exit_flags=bitor(exit_flags,4);
            newParams=[];
            return;
        end
    else	% freeze the small models ( dont change their cov mat )
        if ~all(ismember(verySmall,frozen_models))
            disp(sprintf('Model %d has been freezed\n',setdiff(verySmall,frozen_models)));
            exit_flags=bitor(exit_flags,32);
            frozen_models=union(frozen_models,verySmall);
        end
    end
end


if late_oracle
    newWeights= approximateWeights2(naiveWeights,EMParams);
end

% the new gaussian params

for j=1:k
    
    % if this is the last model and background model flag is on, then don't
    % update this model's parameters (only the weight) - this is the background model.
    if EMParams.BgrModel & j==k
        newParams{j,3}=newWeights(j);
        newParams{j,2}=param{j,2};
        newParams{j,1}=param{j,1};
        break;
    end
    
    if EMParams.remove_models_flag==0 & ismember(j,verySmall)	% check if model should be freezed
        newParams{j,3}=newWeights(j);	% the model's weight isn't freezed
        
        %modified to handle weights:
        if(~isempty(w))
            newParams{j,1}= w.*obsProbabilities(:,j)'*data/normalizer(j); % the model's mean isn't freezed.
        else
            newParams{j,1}= obsProbabilities(:,j)'*data/normalizer(j); % the model's mean isn't freezed.
        end
        
        newParams{j,2}=param{j,2};		% the model's covmat is freezed ( same as the old one )
        newParams{j,4}=param{j,4};	% the labeled staff is freezed.
    else		% set all the model's new parameters :
        
        % labeled
        newParams{j,4}=zeros(1,k);
        
        for i=1:EMParams.num_colors
            newParams{j,4}(EMParams.colors(i))=...
                sum(obsProbabilities(EMParams.lab_mat(i,3:end-EMParams.lab_mat(i,1)),j));
        end
        if sum(newParams{j,4},2)>0
            newParams{j,4}=newParams{j,4}./sum(newParams{j,4},2);
        else
            newParams{j,4}=zeros(1,k);
        end
        % %%%%%%%%%%%%%% end labeled
        
        
        newParams{j,3}=newWeights(j);
        
        % new centers - regular EM update:
        
        %modified to handle weights:
        if(~isempty(w))
            newParams{j,1}= w.*obsProbabilities(:,j)'*data/normalizer(j); % the model's mean isn't freezed.
        else
            newParams{j,1}= obsProbabilities(:,j)'*data/normalizer(j);
        end
        
        if ~EMParams.fixed_covmat      % new cov matrices
            % substract new mean from each observable data point
            centeredData= data - ones(n,1)*newParams{j,1};
            
            %modify to handle weights:
            if(~isempty(w))
                probMat= (ones(d,1)*(w.*obsProbabilities(:,j)'));
            else
                probMat= (ones(d,1)*(obsProbabilities(:,j)'));
            end      
            
            covMat= ( probMat.*centeredData')*centeredData;
            % check if multiple covariance matrix assesment
            if ( EMParams.single_cov_mat_flag==0)
                
                
                newParams{j,2}= covMat./normalizer(j);
                
                if( EMParams.diagonal_covmat_flag  == 1 ) %remove all off diagonal elements
                    newParams{j,2}= newParams{j,2}.*eye(size(covMat));
                end
                
            else
                %single cov mat:
                newParams{1,2}= newParams{1,2} + covMat; 
            end
        else
            if(EMParams.single_cov_mat_flag==0)
                newParams{j,2}= param{j,2};
            else
                newParams{1,2}= param{1,2};
            end
        end
    end
end

%check if need to normalize single cov matrix.
if( (EMParams.single_cov_mat_flag==1)&(~EMParams.fixed_covmat) )
    newParams{1,2}= newParams{1,2}./n; %noramlize by total number of
    %data points.
end

%check if need to remove off diagonal elements in diagonal covmat case:
if(EMParams.diagonal_covmat_flag ==1)
    if(~EMParams.single_cov_mat_flag)
        for i=1:k
            newParams{i,2}= newParams{i,2}.*eye(size(covMat));
            
            %if(rank(newParams{i,2})<d)
            % basic suggestio
            epsilon= 3e-5; 
            %newParams{i,2}= newParams{i,2}+ epsilon.*(eye(size(covMat)));
            newParams{i,2}= newParams{i,2}+ epsilon.*TOTAL_COV;
            %end
        end
    else
        if(~EMParams.fixed_covmat)
            newParams{1,2}= newParams{1,2}.*eye(size(covMat));
            epsilon= 3e-5; 
            newParams{1,2}= newParams{1,2}+ epsilon.*TOTAL_COV;
            
        end
    end
end


if EMParams.fixed_covmat
    for i=1:k
        newParams{i,2}=param{i,2};
    end
end


